from enum import Enum


class SocialLoginProvider(Enum):
    GOOGLE = "Google"
    GITHUB = "GitHub"
    MICROSOFT = "Microsoft"
    SLACK = "Slack"
    LINKEDIN = "LinkedIn"
    SALESFORCE = "Salesforce"
    XERO = "Xero"
    QUICKBOOKS_ONLINE = "QuickBooks Online"


class SamlLoginProvider(Enum):
    GOOGLE = "Google"
    RIPPLING = "Rippling"
    ONELOGIN = "OneLogin"
    JUMPCLOUD = "JumpCloud"
    OKTA = "Okta"
    AZURE = "Azure"
    DUO = "Duo"
    GENERIC = "Generic"


class PasswordLoginMethod:
    def __init__(self):
        self.login_method = "password"

    def __eq__(self, other):
        return isinstance(other, PasswordLoginMethod)


class MagicLinkLoginMethod:
    def __init__(self):
        self.login_method = "magic_link"

    def __eq__(self, other):
        isinstance(other, MagicLinkLoginMethod)


class SocialSsoLoginMethod:
    def __init__(self, provider: SocialLoginProvider):
        self.login_method = "social_sso"
        self.provider = provider

    def __eq__(self, other):
        if isinstance(other, SocialSsoLoginMethod):
            return (
                self.login_method == other.login_method
                and self.provider == other.provider
            )
        return False


class EmailConfirmationLinkLoginMethod:
    def __init__(self):
        self.login_method = "email_confirmation_link"

    def __eq__(self, other):
        return isinstance(other, EmailConfirmationLinkLoginMethod)


class SamlSsoLoginMethod:
    def __init__(self, provider: SamlLoginProvider, org_id: str):
        self.login_method = "saml_sso"
        self.provider = provider
        self.org_id = org_id

    def __eq__(self, other):
        if isinstance(other, SamlSsoLoginMethod):
            return (
                self.login_method == other.login_method
                and self.provider == other.provider
                and self.org_id == other.org_id
            )
        return False


class ImpersonationLoginMethod:
    def __init__(self):
        self.login_method = "impersonation"

    def __eq__(self, other):
        return isinstance(other, ImpersonationLoginMethod)


class GeneratedFromBackendApiLoginMethod:
    def __init__(self):
        self.login_method = "generated_from_backend_api"

    def __eq__(self, other):
        return isinstance(other, GeneratedFromBackendApiLoginMethod)


class UnknownLoginMethod:
    def __init__(self):
        self.login_method = "unknown"

    def __eq__(self, other):
        return isinstance(other, UnknownLoginMethod)


def to_login_method(d={}):
    if "login_method" not in d:
        return UnknownLoginMethod()
    if d["login_method"] == "password":
        return PasswordLoginMethod()
    if d["login_method"] == "magic_link":
        return MagicLinkLoginMethod()
    if d["login_method"] == "social_sso":
        return SocialSsoLoginMethod(
            provider=SocialLoginProvider(d["provider"]),
        )
    if d["login_method"] == "email_confirmation_link":
        return EmailConfirmationLinkLoginMethod()
    if d["login_method"] == "saml_sso":
        return SamlSsoLoginMethod(
            provider=SamlLoginProvider(d["provider"]),
            org_id=d["org_id"],
        )
    if d["login_method"] == "impersonation":
        return ImpersonationLoginMethod()
    if d["login_method"] == "generated_from_backend_api":
        return GeneratedFromBackendApiLoginMethod()
    return UnknownLoginMethod()



import requests
from propelauth_py.api import _ApiKeyAuth, _is_valid_id
from propelauth_py.errors import BadRequestException, UserNotFoundException

ENDPOINT_PATH = "/api/backend/v1/access_token"


def _create_access_token(auth_url, integration_api_key, user_id, duration_in_minutes):
    if not _is_valid_id(user_id):
        raise UserNotFoundException()

    url = auth_url + ENDPOINT_PATH
    json = {"user_id": user_id, "duration_in_minutes": duration_in_minutes}
    response = requests.post(url, json=json, auth=_ApiKeyAuth(integration_api_key))
    if response.status_code == 401:
        raise ValueError("integration_api_key is incorrect")
    elif response.status_code == 400:
        raise BadRequestException(response.json())
    elif response.status_code == 403:
        raise UserNotFoundException()
    elif response.status_code == 404:
        raise RuntimeError("Access token creation is not enabled")
    elif not response.ok:
        raise RuntimeError("Unknown error when creating access token")

    return response.json()



import requests

from propelauth_py.api import _ApiKeyAuth
from propelauth_py.errors import BadRequestException

ENDPOINT_PATH = "/api/backend/v1/magic_link"


####################
#       POST       #
####################
def _create_magic_link(
    auth_url,
    integration_api_key,
    email,
    redirect_to_url=None,
    expires_in_hours=None,
    create_new_user_if_one_doesnt_exist=None,
    user_signup_query_parameters=None,
):
    url = auth_url + ENDPOINT_PATH
    json = {"email": email}
    if redirect_to_url is not None:
        json["redirect_to_url"] = redirect_to_url
    if expires_in_hours is not None:
        json["expires_in_hours"] = expires_in_hours
    if user_signup_query_parameters is not None:
        json["user_signup_query_parameters"] = user_signup_query_parameters
    if create_new_user_if_one_doesnt_exist is not None:
        json[
            "create_new_user_if_one_doesnt_exist"
        ] = create_new_user_if_one_doesnt_exist

    response = requests.post(url, json=json, auth=_ApiKeyAuth(integration_api_key))
    if response.status_code == 401:
        raise ValueError("integration_api_key is incorrect")
    elif response.status_code == 400:
        raise BadRequestException(response.json())
    elif not response.ok:
        raise RuntimeError("Unknown error when creating magic link")

    return response.json()
